package com.ylcm.sys.mapper;

import java.util.List;
import java.util.Map;

import com.ylcm.sys.domain.Permission;


public interface PermissionMapper {

	/**
	 * 新增权限
	 * 
	 * @param permission
	 *            权限信息
	 */
	void addPermission(Permission permission);

	/**
	 * 更新权限
	 * 
	 * @param permission
	 *            权限
	 */
	void updPermission(Permission permission);

	/**
	 * 删除权限信息
	 * 
	 * @param permissionId
	 *            权限标识
	 */
	void delPermission(int permissionId);

	/**
	 * 查询父级权限列表
	 * 
	 * @return list
	 */
	List<Permission> getMenuList();

	/**
	 * 获取父级菜单权限信息(含子菜单)
	 * 
	 * @return 所有权限信息
	 */
	List<Permission> getAllMenus();

	/**
	 * 根据权限标识获取权限信息
	 * 
	 * @param permissionId
	 *            权限标识
	 * @return 权限信息
	 */
	Permission getPermissionById(int permissionId);

	/**
	 * 根据角色获取父级菜单权限信息(含子菜单)
	 * 
	 * @param roleId
	 *            角色id
	 * @return 权限列表
	 */
	List<Permission> getMenusByRoleId(int roleId);

	/**
	 * 根据角色获取所有权限信息
	 * 
	 * @param roleId
	 *            角色id
	 * @return 权限列表
	 */
	List<Permission> getPermissionsByRoleId(int roleId);

	/**
	 * 根据角色获取所有权限信息（登录）
	 * 
	 * @param roleId
	 *            角色id
	 * @return 权限列表
	 */
	List<Permission> getByRoleIdForLogin(int roleId);

	/**
	 * 根据条件查询权限总数
	 * 
	 * @param map
	 *            权限类型 及 父权限标识
	 * @return 权限总数
	 */
	@SuppressWarnings("rawtypes")
	int getRowCount(Map map);

	/**
	 * 根据条件查询权限列表
	 * 
	 * @param map
	 *            权限类型 及 父权限标识
	 * @return 权限集合
	 */
	@SuppressWarnings("rawtypes")
	List<Permission> getPermissionList(Map map);

	/**
	 * 查找当前权限是否关联角色
	 * 
	 * @param permissionId
	 *            权限标识
	 * @return 关联角色的权限
	 */
	Permission getRolePermById(int permissionId);

	/**
	 * 删除权限信息(只删除未关联角色的权限)
	 * 
	 * @param ids
	 *            权限标识集合
	 */
	void delPermissionsByIds(String[] ids);

	/**
	 * 唯一性判断
	 * 
	 * @param map
	 *            条件
	 * @return 数量
	 */
	@SuppressWarnings("rawtypes")
	int getCheckPermission(Map map);

	/**
	 * 删除角色权限关联信息(根据角色id)
	 * 
	 * @param 角色id
	 */
	void delRolePermissionsById(int roleId);

	/**
	 * 添加角色权限关联信息
	 * 
	 * @param map
	 */
	@SuppressWarnings("rawtypes")
	void addRolePermission(Map map);
	
	List<Permission> getAllFunctions();
	
	/**
	 * 根据父权限获取所有菜单子权限
	 * 
	 * @param map
	 */
	List<Permission> getPermissionsByPid(int pid);
}