package com.ylcm.sys.service;

import java.util.List;
import java.util.Map;

import com.ylcm.sys.domain.Permission;


public interface PermissionService {

	/**
	 * 新增权限
	 * 
	 * @param permission
	 *            权限信息
	 */
	void addPermission(Permission permission);

	/**
	 * 更新权限
	 * 
	 * @param permission
	 *            权限
	 */
	void updPermission(Permission permission);

	/**
	 * 删除权限信息
	 * 
	 * @param permissionId
	 *            权限标识
	 */
	void delPermission(int permissionId);

	/**
	 * 查询父级权限列表
	 * 
	 * @return list
	 */
	public List<Permission> getMenuList();

	/**
	 * 获取父级菜单权限信息(含子菜单)
	 * 
	 * @return 获取父级菜单权限信息(含子菜单)
	 */
	List<Permission> getAllMenus();

	/**
	 * 根据权限标识获取权限信息
	 * 
	 * @param permissionId
	 *            权限标识
	 * @return 权限信息
	 */
	Permission getPermissionById(int permissionId);

	/**
	 * 根据角色获取父级菜单权限信息 (含子菜单)
	 * 
	 * @return 权限信息
	 */
	List<Permission> getMenusByRoleId(int roleId);

	/**
	 * 根据角色获取所有权限信息
	 * 
	 * @param roleId
	 *            角色id
	 * @return 权限列表
	 */
	List<Permission> getPermissionsByRoleId(int roleId);

	/**
	 * 根据条件查询权限总数
	 * 
	 * @param name
	 *            名称
	 * @param code
	 *            编码
	 * @param type
	 *            类型
	 * @param pageIndex
	 *            页码
	 * @param pageSize
	 *            页面大小
	 * @return 总数
	 */
	int getRowCount(String name, String code, String type, int pageIndex,
			int pageSize);

	/**
	 * 根据条件查询权限列表
	 * 
	 * @param name
	 *            名称
	 * @param code
	 *            编码
	 * @param type
	 *            类型
	 * @param pageIndex
	 *            页码
	 * @param pageSize
	 *            页面大小
	 * @return 权限列表
	 */
	List<Permission> getPermissionList(String name, String code, String type,
			int pageIndex, int pageSize);

	/**
	 * 查找当前权限是否关联角色
	 * 
	 * @param permissionId
	 *            权限标识
	 * @return 关联角色的权限
	 */
	Permission getRolePermById(int permissionId);

	/**
	 * 删除权限信息(只删除未关联角色的权限)
	 * 
	 * @param ids
	 *            权限标识集合
	 */
	void delPermissionsByIds(String[] ids);

	/**
	 * 唯一性判断
	 * 
	 * @param map
	 *            条件
	 * @return 数量
	 */
	@SuppressWarnings("rawtypes")
	int getCheckPermission(Map map);

	/**
	 * 删除角色权限关联信息(根据角色id)
	 * 
	 * @param 角色id
	 */
	void delRolePermissionsById(int roleId);

	/**
	 * 添加多条角色权限关联信息（根据角色）
	 * 
	 * @param permissionIds
	 *            权限ids
	 * @param roleId
	 *            角色id
	 */
	void addRolePermissions(String permissionIds, int roleId);
	/**
	 * 获取所有功能菜单
	 * @return
	 */
	List<Permission> getAllFunctions(List<Permission> permissionList);
	
}
